/** \file user_terms.h     Functions for user-defined energy terms.
 *
 *             Part of MODELLER, Copyright(c) 1989-2007 Andrej Sali
 */

#ifndef __MOD_USER_TERM_H
#define __MOD_USER_TERM_H

#include "mod_types.h"

#ifdef __cplusplus
extern "C" {
#endif

/** Callback function to evaluate a user-defined energy term. Given the atom
    indices, it should return the total energy and, if deriv is True, also
    the first derivatives with respect to the x/y/z atom coordinates. */
typedef int (*cb_teval)(void *data, const struct model *model, mbool deriv,
                        const int *atind, int n_atind, float *e_term,
                        float *dvx, float *dvy, float *dvz);

/** Create a new user-defined energy term before indx, using the passed
    callback functions */
void new_energy_term(struct energy_data *edat, int indx, cb_teval evalfunc,
                     cb_free freefunc, void *evaldata, int physical_type);

/** Remove the energy term at position indx */
void del_energy_term(struct energy_data *edat, int indx);

#ifdef __cplusplus
}
#endif
#endif  /* __MOD_USER_TERM_H */
